import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Date;

import org.apache.log4j.Logger;
import org.jboss.serial.io.JBossObjectInputStream;
import org.jboss.serial.io.JBossObjectOutputStream;


public class Driver {

	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {

		byte[] bytes ;
		long startingTime;
		long endTime;
		boolean deserialize = false;
		boolean createSerializedObjFile = false ;
		boolean loadTest = true;
		
		if(deserialize){
			bytes = FileUtility.readFromFile(FileUtility.DEFAULT_FILE_NAME);
			startingTime = System.currentTimeMillis();
			System.out.println("Starting Time : " + (new Date(startingTime)));
			TestClass deserializeObj = (TestClass) getObjectFromByteArray(bytes);
			endTime = System.currentTimeMillis();
			System.out.println("End Time : " + (new Date(endTime)));
			System.out.println("Time Difference: " + (endTime  - startingTime)/1000 + " milliseconds");
			System.out.println(deserializeObj);
			
			bytes = FileUtility.readFromFile(FileUtility.DEFAULT_FILE_NAME + "1");
			startingTime = System.currentTimeMillis();
			System.out.println("Starting Time : " + (new Date(startingTime)));
			deserializeObj = (TestClass) getObjectFromByteArrayJBoss(bytes);
			endTime = System.currentTimeMillis();
			System.out.println("End Time : " + (new Date(endTime)));
			System.out.println("Time Difference: " + (endTime  - startingTime)/1000 + " milliseconds");
			System.out.println(deserializeObj);
		}

		if(createSerializedObjFile){
			TestClass temp = new TestClass();
			bytes = getByteArrayFromObject(temp);
			FileUtility.writeToFile(FileUtility.DEFAULT_FILE_NAME, bytes);
			
			bytes = getByteArrayFromObjectJBoss(temp);
			FileUtility.writeToFile(FileUtility.DEFAULT_FILE_NAME + "1",  bytes);
		}
		
		if(loadTest){
			TestClass obj[] = new TestClass[500000];
			for(int i = 0; i < obj.length; i++)
				obj[i] = new TestClass();
			
			startingTime = System.currentTimeMillis();
			System.out.println("Starting Time : " + (new Date(startingTime)));
			for (TestClass testClass : obj) {
				getByteArrayFromObject(testClass);
			}
			endTime = System.currentTimeMillis();
			System.out.println("End Time : " + (new Date(endTime)));
			System.out.println("Time Difference: " + (endTime  - startingTime)/1000 + " milliseconds");

			startingTime = System.currentTimeMillis();
			System.out.println("Starting Time : " + (new Date(startingTime)));
			for (TestClass testClass : obj) {
				getByteArrayFromObject(testClass);
			}
			endTime = System.currentTimeMillis();
			System.out.println("End Time : " + (new Date(endTime)));
			System.out.println("Time Difference: " + (endTime  - startingTime)/1000 + " milliseconds");
			
			startingTime = System.currentTimeMillis();
			System.out.println("Starting Time : " + (new Date(startingTime)));
			for (TestClass testClass : obj) {
				getByteArrayFromObjectJBoss(testClass);
			}
			endTime = System.currentTimeMillis();
			System.out.println("End Time : " + (new Date(endTime)));
			System.out.println("Time Difference: " + (endTime  - startingTime)/1000 + " milliseconds");

			startingTime = System.currentTimeMillis();
			System.out.println("Starting Time : " + (new Date(startingTime)));
			for (TestClass testClass : obj) {
				getByteArrayFromObjectJBoss(testClass);
			}
			endTime = System.currentTimeMillis();
			System.out.println("End Time : " + (new Date(endTime)));
			System.out.println("Time Difference: " + (endTime  - startingTime)/1000 + " milliseconds");

		}
		
		System.out.println(" -----  THE END  ----- ");
	}
	
	/**
	 * Converts an object to a serialized byte array.
	 *
	 * @param obj Object to be converted.
	 * @return byte[] Serialized array representing the object.
	 */
	public static byte[] getByteArrayFromObjectJBoss(Object obj) {
	    byte[] result = null;
	        
	    try {
	        ByteArrayOutputStream baos = new ByteArrayOutputStream();
	        ObjectOutputStream oos = new JBossObjectOutputStream(baos);
	        oos.writeObject(obj);
	        oos.flush();
	        oos.close();
	        baos.close();
	        result = baos.toByteArray();
	    } catch (IOException ioEx) {
	        Logger.getLogger("UtilityMethods").error("Error converting object to byteArray", ioEx);
	    }
	         
	    return result;
	}
	 
	/**
	 * Utility method to un-serialize objects from byte arrays.
	 *
	 * @param bytes The input byte array.
	 * @return The output object.
	 */
	public static Object getObjectFromByteArrayJBoss(byte[] bytes) {
	    Object result = null;
	  
	    try {
	        ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
	        ObjectInputStream ois = new JBossObjectInputStream(bais);
	        result = ois.readObject();
	        ois.close();
	    } catch (IOException ioEx) {
	        Logger.getLogger("UtilityMethods").error("Unable to deserialize object from byte array.",
	            ioEx);
	    } catch (ClassNotFoundException cnfEx) {
	        Logger.getLogger("UtilityMethods").error("No corresponding class for byte array.",
	            cnfEx);
	    }
	   
	    return result;
	}
	
	/**
	 * Converts an object to a serialized byte array.
	 *
	 * @param obj Object to be converted.
	 * @return byte[] Serialized array representing the object.
	 */
	public static byte[] getByteArrayFromObject(Object obj) {
	    byte[] result = null;
	        
	    try {
	        ByteArrayOutputStream baos = new ByteArrayOutputStream();
	        ObjectOutputStream oos = new ObjectOutputStream(baos);
	        oos.writeObject(obj);
	        oos.flush();
	        oos.close();
	        baos.close();
	        result = baos.toByteArray();
	    } catch (IOException ioEx) {
	        Logger.getLogger("UtilityMethods").error("Error converting object to byteArray", ioEx);
	    }
	         
	    return result;
	}
	 
	/**
	 * Utility method to un-serialize objects from byte arrays.
	 *
	 * @param bytes The input byte array.
	 * @return The output object.
	 */
	public static Object getObjectFromByteArray(byte[] bytes) {
	    Object result = null;
	  
	    try {
	        ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
	        ObjectInputStream ois = new ObjectInputStream(bais);
	        result = ois.readObject();
	        ois.close();
	    } catch (IOException ioEx) {
	        Logger.getLogger("UtilityMethods").error("Unable to deserialize object from byte array.",
	            ioEx);
	    } catch (ClassNotFoundException cnfEx) {
	        Logger.getLogger("UtilityMethods").error("No corresponding class for byte array.",
	            cnfEx);
	    }
	   
	    return result;
	}
	

}
